﻿Public Class Complex
    Public Re As Double = 0
    Public Im As Double = 0

    ' Constructors.
    Public Sub New(ByVal real As Double, ByVal imaginary As Double)
        Re = real
        Im = imaginary
    End Sub
    Public Sub New(ByVal real As Double)
        Re = real
        Im = 0
    End Sub

    ' Parse a String of the form R + Ii.
    Public Shared Function Parse(ByVal string_value As String) As Complex
        ' Remove spaces and the "i".
        string_value = string_value.Replace(" ", "")
        string_value = string_value.ToLower().Replace("i", "")

        ' Find the + or - between the parts.
        Dim plus_or_minus() As Char = {"+"c, "-"c}
        Dim pos As Integer = string_value.IndexOfAny(plus_or_minus)
        If (pos = 0) Then
            ' Skip the leading +/-.
            pos = string_value.IndexOfAny(plus_or_minus, 1)
        End If

        ' Get the real and imaginary parts.
        Dim re As Double = Double.Parse(string_value.Substring(0, pos))
        Dim im As Double = Double.Parse(string_value.Substring(pos))

        Return New Complex(Re, Im)
    End Function

    ' Operators.
    ' Unary -.
    Public Shared Operator -(ByVal c1 As Complex) As Complex
        Return New Complex(-c1.Re, -c1.Im)
    End Operator

    ' Binary +.
    Public Shared Operator +(ByVal c1 As Complex, ByVal c2 As Complex) As Complex
        Return New Complex(c1.Re + c2.Re, c1.Im + c2.Im)
    End Operator

    ' Binary *.
    Public Shared Operator *(ByVal c1 As Complex, ByVal c2 As Complex) As Complex
        Return New Complex( _
            c1.Re * c2.Re - c1.Im * c2.Im, _
            c1.Re * c2.Im + c1.Im * c2.Re)
    End Operator

    ' Binary -.
    Public Shared Operator -(ByVal c1 As Complex, ByVal c2 As Complex) As Complex
        Return New Complex(c1.Re - c2.Re, c1.Im - c2.Im)
    End Operator

    ' Binary /.
    Public Shared Operator /(ByVal c1 As Complex, ByVal c2 As Complex) As Complex
        Dim denominator As Double = c2.Re * c2.Re + c2.Im * c2.Im
        Dim re As Double = _
            (c1.Re * c2.Re + c1.Im * c2.Im) / _
            denominator
        Dim im As Double = _
            (c1.Im * c2.Re - c1.Re * c2.Im) / _
            denominator
        Return New Complex(Re, Im)
    End Operator

    ' Convert between double and Complex.
    Public Shared Widening Operator CType(ByVal re As Double) As Complex
        Return New Complex(re, 0)
    End Operator

    Public Shared Narrowing Operator CType(ByVal c1 As Complex) As Double
        Return c1.Magnitude
    End Operator

    ' Return the number's magnitude.
    Public ReadOnly Property Magnitude() As Double
        Get
            Return (Math.Sqrt(Re * Re + Im * Im))
        End Get
    End Property

    ' Return the number's magnitude squared.
    Public ReadOnly Property MagnitudeSquared() As Double
        Get
            Return (Re * Re + Im * Im)
        End Get
    End Property

    ' Unary +.
    Public Shared Operator +(ByVal c1 As Complex) As Complex
        Return New Complex(c1.Re, c1.Im)
    End Operator

    ' < and >.
    Public Shared Operator <(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return (c1.Magnitude < c2.Magnitude)
    End Operator
    Public Shared Operator >(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return (c1.Magnitude > c2.Magnitude)
    End Operator

    ' <= and >=.
    Public Shared Operator <=(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return (c1.Magnitude <= c2.Magnitude)
    End Operator
    Public Shared Operator >=(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return (c1.Magnitude >= c2.Magnitude)
    End Operator

    ' = and <>.
    Public Shared Operator =(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return (c1.Re = c2.Re AndAlso c1.Im = c2.Im)
    End Operator
    Public Shared Operator <>(ByVal c1 As Complex, ByVal c2 As Complex) As Boolean
        Return Not (c1 = c2)
    End Operator
    Public Overrides Function Equals(ByVal obj As Object) As Boolean
        Dim c1 As Complex = DirectCast(obj, Complex)
        Return (Me.Re = c1.Re) AndAlso (Me.Im = c1.Im)
    End Function
    Public Overrides Function GetHashCode() As Integer
        Return Re.GetHashCode() Or Im.GetHashCode()
    End Function

    ' Return the number formatted as a string.
    Public Overrides Function ToString() As String
        If (Im = 0) Then Return String.Format("{0}", Re)
        If (Re = 0) Then Return String.Format("{0}i", Im)
        If (Im < 0) Then Return String.Format("{0} - {1}i", Re, Math.Abs(Im))
        Return String.Format("{0} + {1}i", Re, Im)
    End Function
    Public Overloads Function ToString(ByVal format As String) As String
        If (Im = 0) Then Return String.Format("{0:" + format + "}", Re)
        If (Re = 0) Then Return String.Format("{0:" + format + "}i", Im)
        If (Im < 0) Then Return String.Format("{0:" + format + "} - {1:" + format + "}i", Re, Math.Abs(Im))
        Return String.Format("{0:" + format + "} + {1:" + format + "}i", Re, Im)
    End Function
End Class
